/*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
// (C) PicoPeta Simputers Pvt. Ltd. (2003)
// Author: Vivek K S

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <stdlib.h>
#include <string.h>
#include "alchemy.h"
#include "basic_window.h"
#include "star_window.h"
#include <image.h>
#include "utils.h"
#include <font.h>

struct _star_menu_list *star_menu_list = NULL;

px_widget *star_create_listbox(wm *wm, px_widget *frame, int *width, int *height, int *n);

// Create a new window showing the list of keyboard layouts
window *new_star_window(wm *wm)
{
	Window frame_window;
	Display *dpy=wm->display;
	ac_theme *theme=wm->current_theme;
	XSetWindowAttributes win_attr;
	window *w;
	int n, width, height;
	px_widget *listbox;

	win_attr.override_redirect = True;
	win_attr.background_pixel = wm->black_pixel;
	win_attr.border_pixel = wm->black_pixel;
	win_attr.event_mask = ButtonPressMask|ButtonReleaseMask|ExposureMask|
		EnterWindowMask|LeaveWindowMask|PointerMotionMask;

	// Create with small size, Resize and Reposition after getting
	// the actual size and position
	frame_window = XCreateWindow(dpy, wm->root, 0,0, 1, 1, 0,
			CopyFromParent, CopyFromParent, CopyFromParent, 
			CWOverrideRedirect|CWBackPixel|CWBorderPixel|CWEventMask,
			&win_attr);

	// Allocate structures, there is no separate app_window here
	w = new_basic_window(wm, frame_window, MENU_WIN);
	w->frame_window = frame_window;
	w->app_window = frame_window;
	
	// The topbar frame is used to hold the listbox widget
	w->topbar_frame=new_toplevel_frame(frame_window, 0, 0, 1, 1, NULL);

	// Create the listbox and figure out dimensions
	listbox = star_create_listbox(wm, w->topbar_frame, &width, &height, &n);
	listbox->redraw(listbox);
	w->topbar_frame->set_size(w->topbar_frame, width, height);
	
	// Move/Resize window according to calculated dimensions
	XMoveResizeWindow(dpy, frame_window, theme->get_menu_x(theme) - 5,
			theme->get_topbar_height(theme),
			width, height);
	XSetWindowBorderWidth(dpy, frame_window, 1 );
	
	w->destroy = star_window_destroy;
	
	w->topbar_frame->redraw(w->topbar_frame);
	w->topbar_frame->show(w->topbar_frame);
	
	window_set_state(w, NormalState);

	// Show the window
	XMapWindow(dpy, w->frame_window);
	wm->is_menu_mapped = True;

	// Grab pointer
	XGrabPointer(dpy, wm->root, True, (ButtonPressMask|ButtonReleaseMask),
			GrabModeAsync, GrabModeAsync, None, wm->normal_cursor, CurrentTime);
	
	XSync(wm->display, False);

	return w;
}

static void *menu_background;
static char menu_bg_type;
static Bool menu_initialized=False;

// Create the listbox of pixmaps
px_widget *star_create_listbox(wm *wm, px_widget *frame, int *width, int *height, int *n)
{
	int i;
	px_widget *list;
	struct _star_menu_list *node = NULL;	
	const int MAXSIZE = 1024;
	px_color transp_color, active_col;
	ac_theme *theme=wm->current_theme;
	
	transp_color.type = TRANSPARENT;
	active_col.type =SOLID;
	widget_xcolor_from_str(&(active_col.first_color), "black");

	list = new_widget(LISTBOX, frame, 0,0,1,1, NULL);
	*width = 0;
	*height = list->specific.listbox.inter_widget_space;
	*n = 0;

	if(!menu_initialized) {
		menu_background = theme->get_menu_background( theme,
				&menu_bg_type);
		menu_initialized = True;
	}

	if(menu_bg_type) {
		list->set_background_color(list, (px_color *) menu_background);
	} else {
		list->set_background_pixmap_from_pixmap(list, (px_pixmap *) menu_background);
	}

   	if (star_menu_list == NULL)
	{
		char *directory = theme->theme_path;
		char *filename = (char*)malloc(sizeof(char) * (strlen(directory) + 15));
		char *ctemp = (char*)malloc(sizeof(char) * MAXSIZE);
		FILE* fileptr;
	   
		strcpy(filename, directory);
		strcat(filename, "starmenu.cfg");

		fileptr = fopen(filename,"rt");
		free(filename);
		filename = NULL;

		if (fileptr != NULL)
		{
			struct _star_menu_list* last_node = NULL;
			
			while(fgets(ctemp,MAXSIZE,fileptr) != NULL)
			{
				struct _star_menu_list* node = (struct _star_menu_list*)malloc(sizeof(struct _star_menu_list));
				char* colon_1, *colon_2=NULL;
				colon_1 = strchr(ctemp, ':');

				node->command = NULL;
				node->pixmap = NULL;
				node->active_pixmap = NULL;
				node->next = NULL;

				ctemp[strlen(ctemp)-1] = '\0';
				if (colon_1 != NULL)
				{
					(*colon_1) = '\0';
					
					colon_2 = strchr(colon_1+1, ':');

				}

				if (colon_2 != NULL)
				{
					(*colon_2) = '\0';
				}

				// Set the command to execute.
				node->command = (char*)malloc(sizeof(char) * (strlen(ctemp) + 1));
				strcpy(node->command, ctemp);
				
				// Set the normal pixmap

				if (colon_1 != NULL)
				{
					char* filename;
					filename = (char*)malloc(sizeof(char) * (strlen(directory) + strlen(colon_1+1) + 1));

					strcpy(filename, directory);
					strcat(filename, colon_1+1);
					
					node->pixmap = (px_pixmap*)malloc(sizeof(px_pixmap));
					image_initialize_pixmap(node->pixmap);
					if (!image_load_file(wm->root, node->pixmap, filename))
					{
						image_destroy(node->pixmap);
						free(node->pixmap);
						node->pixmap = NULL;
					}
					free(filename);
				}
				
				if (colon_2 != NULL)
				{
					char* filename;
					filename = (char*)malloc(sizeof(char) * (strlen(directory) + strlen(colon_2+1) + 1));

					strcpy(filename, directory);
					strcat(filename, colon_2+1);
					
					node->active_pixmap = (px_pixmap*)malloc(sizeof(px_pixmap));
					image_initialize_pixmap(node->active_pixmap);
					if (!image_load_file(wm->root, node->active_pixmap, filename))
					{
						image_destroy(node->active_pixmap);
						free(node->active_pixmap);
						node->active_pixmap = NULL;
					}

					free(filename);
				}				
				node->next = NULL;

				if (last_node != NULL)
					last_node->next = node;
				else
					star_menu_list = node;

				last_node = node;
			}
			fclose(fileptr);
		}
		else // This is not a valid category or category files do not exist.
		{
			int twidth, theight;
			px_widget* label;
			char msg[] = "Invalid theme file";
				

			label = new_widget(LABEL, list, 0, 0, 1, 1, NULL);
			label->specific.label.set_wrap_type(label, NO_WRAP);
			if(menu_bg_type) {
				label->set_background_color(label, (px_color *) menu_background);
			}

			list->specific.listbox.set_child(list, label, *n);
			label->specific.label.set_font(label, DEFAULT_FONT);
			label->specific.label.set_normal_color(label, "gray");

			// Set name and callbacks

			label->specific.label.set_text(label, msg);
		
			// Get font metrics
			twidth = font_get_width(label->specific.label.layout.font, msg, strlen(msg));
			theight = font_get_height(label->specific.label.layout.font, msg);

			if(twidth > *width)
				*width = twidth;

			label->height = theight + 4;
			*height += label->height + list->specific.listbox.inter_widget_space;
		}

	}

	node = star_menu_list;

	while (node != NULL)
	{
		int twidth, theight;
		px_widget* button;

	
		if (node->pixmap == NULL)
		{
			twidth = 16;
			theight = 16;
		}
		else
		{
			twidth = node->pixmap->w;
			theight = node->pixmap->h;
		}

		button = new_widget(BUTTON, list, 0, 0, twidth, theight, node->command);
		button->specific.button.set_wrap_type(button, NO_WRAP);
		button->set_background_color(button, &transp_color);
		button->specific.button.set_active_background_color(button, &active_col);
		list->specific.listbox.set_child(list, button, *n);

		if (node->pixmap != NULL)
		{
			button->specific.button.normal = (px_pixmap*)malloc(sizeof(px_pixmap));
			image_initialize_pixmap(button->specific.button.normal);
			image_copy_pixmap(button->win, button->specific.button.normal, node->pixmap);
		}

		if (node->active_pixmap != NULL)
		{
			button->specific.button.active = (px_pixmap*)malloc(sizeof(px_pixmap));
			image_initialize_pixmap(button->specific.button.active);
			image_copy_pixmap(button->win, button->specific.button.active, node->active_pixmap);
		}
		
		button->callbacks[BUTTON_RELEASED] = star_button_released;
		button->callbacks[MOUSE_ENTER] = star_mouse_enter;
		button->callbacks[MOUSE_LEAVE] = star_mouse_leave;
		

		if(twidth > *width)
			*width = twidth;

		button->height = theight;
		*height += button->height + list->specific.listbox.inter_widget_space;

		(*n)++;
		node = node->next;
	}

	*width += 14;
	
	for(i=0; i<list->specific.listbox.no_of_children; i++) {
		px_widget *l = list->specific.listbox.widget_list[i];

		l->set_size(l, *width, l->height);
		l->redraw(l);
	}
	
	list->set_size(list, *width, *height);

	return list;
}

//Deletes the cache.
void star_window_delete_cache()
{
	if (star_menu_list != NULL)
	{
		struct _star_menu_list *node, *next_node;

		node = star_menu_list;
		while (node != NULL)
		{
			next_node = node->next;
			if (node->command != NULL)
				free(node->command);

			image_destroy(node->pixmap);
			image_destroy(node->active_pixmap);
			free(node->pixmap);
			free(node->active_pixmap);
			free(node);
			node = next_node;
		}
		star_menu_list = NULL;
	}
}
	
// Destroy this menu
void star_window_destroy(window *w)
{
	Display *dpy=w->wm->display;

	XUnmapWindow(dpy, w->frame_window);
	XUngrabPointer(dpy, CurrentTime);
	window_set_state(w, WithdrawnState);

	w->wm->is_menu_mapped = False;

	XGrabServer(dpy);
	window_destroy(w);
	XUngrabServer(dpy);
}

// Pointer released; Run application
Bool star_button_released(px_widget *pw, px_event *pxe)
{
	window *w;
	unsigned int len = strlen(pw->tag);
	char* command = (char*)malloc(sizeof(char) * (len + 2));
	char transient_str[100];
	
	w = (window *)pxe->private_data;
	strcpy(command, pw->tag);
	command[len] = '&';
	command[len+1] = '\0';

	sprintf(transient_str, "WM_TRANSIENT_FOR=%lu", w->wm->mapped_window->app_window);
	spawn_child(command, transient_str, NULL);
	free(command);
	w->flags.bits.needs_keyboard = True; // set flag for window to be destroyed

	return True;
}

// Pointer enters the label
Bool star_mouse_enter(px_widget *pw, px_event *pxe)
{
	px_color col;
	col.type =SOLID;
	widget_xcolor_from_str(&(col.first_color), "black");
	pw->set_background_color(pw, &col);
	pw->set_state(pw, ACTIVE);
	pw->redraw(pw);
	pw->show(pw);

	return True;
}

// Pointer leaves the label
Bool star_mouse_leave(px_widget *pw, px_event *pxe)
{
	px_color col;
	col.type = TRANSPARENT;
	//widget_xcolor_from_str(&(col.first_color), "white");
	pw->set_background_color(pw, &col);
	pw->set_state(pw, NORMAL);
	pw->redraw(pw);
	pw->show(pw);

	return True;
}
